"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handler = exports.help = exports.commandNames = exports.cliOptionsTypes = exports.rcOptionsTypes = void 0;
const cli_utils_1 = require("@pnpm/cli-utils");
const config_1 = require("@pnpm/config");
const error_1 = require("@pnpm/error");
const logger_1 = require("@pnpm/logger");
const store_connection_manager_1 = require("@pnpm/store-connection-manager");
const store_path_1 = require("@pnpm/store-path");
const pick_1 = __importDefault(require("ramda/src/pick"));
const render_help_1 = __importDefault(require("render-help"));
const storeAdd_1 = require("./storeAdd");
const storePrune_1 = require("./storePrune");
const storeStatus_1 = require("./storeStatus");
exports.rcOptionsTypes = cliOptionsTypes;
function cliOptionsTypes() {
    return (0, pick_1.default)([
        'registry',
        'store-dir',
        'force',
    ], config_1.types);
}
exports.cliOptionsTypes = cliOptionsTypes;
exports.commandNames = ['store'];
function help() {
    return (0, render_help_1.default)({
        description: 'Reads and performs actions on pnpm store that is on the current filesystem.',
        descriptionLists: [
            {
                title: 'Commands',
                list: [
                    {
                        description: '\
Checks for modified packages in the store. \
Returns exit code 0 if the content of the package is the same as it was at the time of unpacking',
                        name: 'status',
                    },
                    {
                        description: 'Adds new packages to the store. Example: pnpm store add express@4 typescript@2.1.0',
                        name: 'add <pkg>...',
                    },
                    {
                        description: '\
Removes unreferenced (extraneous, orphan) packages from the store. \
Pruning the store is not harmful, but might slow down future installations. \
Visit the documentation for more information on unreferenced packages and why they occur',
                        name: 'prune',
                    },
                    {
                        description: 'If there are alien directories in the store, this command removes them. \
Alien directories are directories/files that were not created by the package manager.',
                        name: 'prune --force',
                    },
                    {
                        description: 'Returns the path to the active store directory.',
                        name: 'path',
                    },
                ],
            },
        ],
        url: (0, cli_utils_1.docsUrl)('store'),
        usages: ['pnpm store <command>'],
    });
}
exports.help = help;
class StoreStatusError extends error_1.PnpmError {
    constructor(modified) {
        super('MODIFIED_DEPENDENCY', '');
        this.modified = modified;
    }
}
async function handler(opts, params) {
    let store;
    switch (params[0]) {
        case 'status':
            return statusCmd(opts);
        case 'path':
            return (0, store_path_1.getStorePath)({
                pkgRoot: opts.dir,
                storePath: opts.storeDir,
                pnpmHomeDir: opts.pnpmHomeDir,
            });
        case 'prune': {
            store = await (0, store_connection_manager_1.createOrConnectStoreController)(opts);
            const storePruneOptions = Object.assign(opts, {
                storeController: store.ctrl,
                storeDir: store.dir,
                removeAlienFiles: opts.force,
                cacheDir: opts.cacheDir,
                dlxCacheMaxAge: opts.dlxCacheMaxAge,
            });
            return (0, storePrune_1.storePrune)(storePruneOptions);
        }
        case 'add':
            store = await (0, store_connection_manager_1.createOrConnectStoreController)(opts);
            return (0, storeAdd_1.storeAdd)(params.slice(1), {
                prefix: opts.dir,
                registries: opts.registries,
                reporter: opts.reporter,
                storeController: store.ctrl,
                tag: opts.tag,
            });
        default:
            return help();
    }
}
exports.handler = handler;
async function statusCmd(opts) {
    const modifiedPkgs = await (0, storeStatus_1.storeStatus)(Object.assign(opts, {
        storeDir: await (0, store_path_1.getStorePath)({
            pkgRoot: opts.dir,
            storePath: opts.storeDir,
            pnpmHomeDir: opts.pnpmHomeDir,
        }),
    }));
    if (!modifiedPkgs || (modifiedPkgs.length === 0)) {
        logger_1.logger.info({
            message: 'Packages in the store are untouched',
            prefix: opts.dir,
        });
        return;
    }
    throw new StoreStatusError(modifiedPkgs);
}
//# sourceMappingURL=store.js.map